import * as effector from 'effector'
import { Unit } from 'effector'
export { ConfigPersist, Done, Fail, Finally, Persist, StorageAdapter } from '..'

interface StorageAdapter {
  <State>(key: string, update: (raw?: any) => any): {
    set(value: State): void
    get(value?: any): State | Promise<State>
  }
  keyArea?: any
}
declare type Done<State> = {
  key: string
  keyPrefix: string
  operation: 'set' | 'get'
  value: State
}
declare type Fail<Err> = {
  key: string
  keyPrefix: string
  operation: 'set' | 'get'
  error: Err
  value?: any
}
declare type Finally<State, Err> =
  | (Done<State> & {
      status: 'done'
    })
  | (Fail<Err> & {
      status: 'fail'
    })
interface ConfigAdapter {
  adapter: StorageAdapter
}
interface ConfigCommon<State, Err = Error> {
  clock?: Unit<any>
  done?: Unit<Done<State>>
  fail?: Unit<Fail<Err>>
  finally?: Unit<Finally<State, Err>>
  pickup?: Unit<any>
  key?: string
  keyPrefix?: string
}

interface ConfigStore<State, Err = Error>
  extends ConfigAdapter,
    ConfigCommon<State, Err> {}
/**
 * `persist` with curried `store`
 */
declare const persist: (
  config?: ConfigStore<any, Error> | undefined
) => <State>(store: effector.Store<State>) => effector.Store<State>

export { ConfigStore, persist }
